#!/bin/sh

#Set scanner
SCANNER=twistcli
# Remove URL scheme as that will be prepended later
TWIST_CLI_H=$(echo "${TWIST_CLI_H}" | sed "s|^https://||")

# Load common utilities
. ./scan_utilities.sh

# Enable debug output
set -x

# Display binary version
twistcli --version

ALWAYS_PASS=true
if test "${TWISTLOCK_CHECK}" = "true"; then ALWAYS_PASS=false; fi

if test "${TWISTLOCK_INFO}" = "true"; then echo "** INFO - THIS TWISTLOCK SCAN IS FOR INFORMATIONAL PURPOSES ONLY AND WILL NOT FAIL YOUR PIPELINE REGARDLESS OF THE SEVERITY OF THE FINDINGS"; fi

# Pull base images in advance
for IMAGE in ${BASE_IMAGES}; do
	if ! docker pull "${IMAGE}" 2>/dev/null; then
		error_msg "Failed pulling base image. Skipping: ${IMAGE}"
	fi
done

SCAN_STATUS=0
for IMAGE in ${SCAN_IMAGE_LIST}; do
	cleanup

	# Logs into ECR if the image is hosted there
	if echo "${IMAGE}" | grep -qi ".dkr.ecr."; then
		login_to_ecr "${IMAGE}"
	fi

	echo "$IMAGE"
	if ! docker pull "${IMAGE}"; then
		error_msg "Failed pulling ${IMAGE}"
		exit 2
	fi

	# Find base image, return first from BASE_IMAGES otherwise
	BASE=$(echo "${BASE_IMAGES}" | cut -d" " -f1)

	BASE_IMAGE_CHECK=true
	# Check if we are scanning an approved base image
	if check_is_re_image "${IMAGE}"; then
		# IMAGE is a base image. Do not diff against base
		BASE_IMAGE_CHECK=false
		BASE=""
	fi

	if ${BASE_IMAGE_CHECK} && test -n "${BASE_IMAGES}"; then
		MAX_COMMON_LAYERS=0
		for BI in ${BASE_IMAGES}; do
			if check_image_base "${IMAGE}" "${BI}"; then
				# note the number of layers in the current base image
				IMAGE_LAYER_COUNT=$(docker inspect --format='{{.RootFS.Layers}}' "${BI}" | wc -c)
				# verify whether the current image overlaps more than theone we have stored
				if [ "$IMAGE_LAYER_COUNT" -gt "$MAX_COMMON_LAYERS" ]; then
					MAX_COMMON_LAYERS="$IMAGE_LAYER_COUNT"
					BASE="${BI}"
				fi
			fi
		done

		export BASE

		# Get the base image for diff-ing
		docker image inspect "${BASE}" >"${REP_PATH_CLI}/base_image.json"
		~/.twistcli/twistcli images scan --address "https://${TWIST_CLI_H}" -u "${TWIST_CLI_U}" -p "${TWIST_CLI_P}" --output-file "${REP_PATH_CLI}/base_image_scan.json" --details "${BASE}" >"${REP_PATH_CLI}/base_image.txt"
	fi
	export IMAGE

	set_image_vars "${IMAGE}"

	docker image inspect "${IMAGE}" >"${REP_PATH_CLI}/target_image.json"

	if ! ~/.twistcli/twistcli images scan --address "https://${TWIST_CLI_H}" -u "${TWIST_CLI_U}" -p "${TWIST_CLI_P}" --output-file "${RES_CLI}.json" --details "${IMAGE}" >"${RES_CLI}.txt"; then
		error_msg "Failed scanning ${IMAGE} with twistcli"
		exit 3
	fi

	sh -x check_fips_openssl.sh "${IMAGE}"
	FIPS_OPENSSL=$?

	sh -x check_fips_java.sh "${IMAGE}"
	FIPS_JAVA=$?

	sh -x check_fips_python.sh "${IMAGE}" "${RES_CLI}.json"
	FIPS_PYTHON=$?

	INFO_OS=""
	INFO_OS=$(get_info_os "${IMAGE}")

	python3 analyser/scanResultApi-twistlock.py \
		-fo ${FIPS_OPENSSL} \
		-fj ${FIPS_JAVA} \
		-fp ${FIPS_PYTHON} \
		-t "${RES_CLI}.json" \
		-b "${REP_PATH_CLI}/base_image_scan.json" \
		-jb "${REP_PATH_CLI}/base_image.json" \
		-jt "${REP_PATH_CLI}/target_image.json" \
		-io "${INFO_OS}" \
		-oh "${RES_CLI}.html" \
		-oj "${JSON_OUTPUT}" || ${ALWAYS_PASS}
	LAST_STATUS=$?

	# If this scan or a previous scan failed, keep the status
	if [ ${SCAN_STATUS} -ne 0 ] || [ ${LAST_STATUS} -ne 0 ]; then
		SCAN_STATUS=1
	fi
done

if test "${TWISTLOCK_INFO}" = "true"; then echo "** INFO - THIS TWISTLOCK SCAN IS FOR INFORMATIONAL PURPOSES ONLY AND WILL NOT FAIL YOUR PIPELINE REGARDLESS OF THE SEVERITY OF THE FINDINGS"; fi

exit ${SCAN_STATUS}
